package com.yizhi.esearch.application.controller.api;

import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.esearch.application.constant.Constant;
import com.yizhi.esearch.application.pojo.biz.EsCourse;
import com.yizhi.esearch.application.pojo.biz.EsLecturer;
import com.yizhi.esearch.application.pojo.biz.EsOfflineCourse;
import com.yizhi.esearch.application.pojo.biz.EsTrainingProject;
import com.yizhi.esearch.application.repository.CourseRepository;
import com.yizhi.esearch.application.repository.LecturerRepository;
import com.yizhi.esearch.application.repository.OfflineCourseRepository;
import com.yizhi.esearch.application.repository.TrainingProjectRepository;
import com.yizhi.esearch.application.util.QueryHelper;
import com.yizhi.util.application.domain.Response;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.sort.SortBuilders;
import org.elasticsearch.search.sort.SortOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.elasticsearch.core.ElasticsearchRestTemplate;
import org.springframework.data.elasticsearch.core.document.Document;
import org.springframework.data.elasticsearch.core.mapping.IndexCoordinates;
import org.springframework.data.elasticsearch.core.query.NativeSearchQueryBuilder;
import org.springframework.data.elasticsearch.core.query.Query;
import org.springframework.data.elasticsearch.core.query.UpdateQuery;
import org.springframework.data.elasticsearch.core.query.UpdateResponse;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

/**
 * @Author: shengchenglong
 * @Date: 2018/12/26 17:54
 */
@Api(tags = "讲师搜索")
@RestController
@RequestMapping("/search/lecturer")
public class LecturerController {

    @Autowired
    private ElasticsearchRestTemplate elasticsearchRestTemplate;
    @Autowired
    private CourseRepository courseRepository;
    @Autowired
    private TrainingProjectRepository trainingProjectRepository;
    @Autowired
    private OfflineCourseRepository offlineCourseRepository;
    @Autowired
    private LecturerRepository lecturerRepository;

    @GetMapping("name/{keyword}/{pageNo}/{pageSize}")
    @ApiOperation(value = "讲师搜索", response = EsLecturer.class)
    public Response<EsLecturer> name(
            @ApiParam(name = "keyword", value = "搜索关键词") @PathVariable("keyword") String keyword,
            @ApiParam(name = "pageNo", value = "第几页") @PathVariable("pageNo") Integer pageNo,
            @ApiParam(name = "pageSize", value = "每页几条") @PathVariable("pageSize") Integer pageSize) {
        return list(keyword, null, pageNo, pageSize);
    }

    @GetMapping("/list")
    @ApiOperation(value = "讲师搜索", response = EsLecturer.class)
    public Response<EsLecturer> list(
            @ApiParam(name = "keyword", value = "搜索关键字") @RequestParam(name = "keyword", required = false) String keyword,
            @ApiParam(name = "classifyId", value = "讲师分类") @RequestParam(name = "classifyId", required = false) String classifyId,
            @ApiParam(name = "pageNo", value = "第几页") @RequestParam("pageNo") Integer pageNo,
            @ApiParam(name = "pageSize", value = "每页几条") @RequestParam("pageSize") Integer pageSize) {

        String siteId = String.valueOf(ContextHolder.get().getSiteId());
        QueryBuilder qbContext = QueryHelper.getCommonQueryBuilderForContext();
        QueryBuilder qbSearch = null;

        if (!StringUtils.isEmpty(keyword)) {
            String relationIds = "";
            // 查出课程的
            List<EsCourse> courses = courseRepository.findBySiteIdEqualsAndNameAndStatusAndDeleted(siteId, keyword, Constant.STATUS_UP, Constant.DELETED_NO);
            if (!CollectionUtils.isEmpty(courses)) {
                for (EsCourse c : courses) {
                    relationIds = relationIds.concat(c.getId().concat(" "));
                }
            }
            // 查出培训项目的
            List<EsTrainingProject> trainingProjects = trainingProjectRepository.findBySiteIdEqualsAndNameAndStatusAndDeleted(siteId, keyword, Constant.STATUS_UP, Constant.DELETED_NO);
            if (!CollectionUtils.isEmpty(trainingProjects)) {
                for (EsTrainingProject tp : trainingProjects) {
                    relationIds = relationIds.concat(tp.getId().concat(" "));
                }
            }
            // 查出线下课程的
            List<EsOfflineCourse> offlineCourses = offlineCourseRepository.findBySiteIdEqualsAndNameAndStatusAndDeleted(siteId, keyword, Constant.STATUS_UP, Constant.DELETED_NO);
            if (!CollectionUtils.isEmpty(offlineCourses)) {
                for (EsOfflineCourse oc : offlineCourses) {
                    relationIds = relationIds.concat(oc.getId().concat(" "));
                }
            }
            QueryBuilder qbForRelationIds = QueryBuilders.boolQuery().must(QueryBuilders.matchQuery("relationIds", relationIds));
            //支持模糊查询，以*为前缀会影响查询性能。
            QueryBuilder qbForLecturerName = QueryBuilders.boolQuery().must(QueryBuilders.matchQuery("lecturerName", keyword));
            qbSearch = QueryBuilders.boolQuery().should(qbForRelationIds).should(qbForLecturerName);
        }

        if (!StringUtils.isEmpty(classifyId)) {
            String[] classifyIds = classifyId.split(",");
            BoolQueryBuilder qb = QueryBuilders.boolQuery();
            for (String id : classifyIds) {
                qb.should(QueryBuilders.matchQuery("classifyId", id));
            }
            if (qbSearch != null) {
                qbSearch = QueryBuilders.boolQuery().must(qbSearch).must(qb);
            } else {
                qbSearch = QueryBuilders.boolQuery().must(qb);
            }
        }

        Query searchQuery = new NativeSearchQueryBuilder()
//                    .addAggregation(getTimeAggreation())
                .withQuery(qbSearch)
                .withFilter(qbContext)
                // page从0开始
                .withPageable(PageRequest.of(pageNo - 1, pageSize))
                // notice: 先按评分倒排，再按业务要求排序
                .withSort(SortBuilders.scoreSort().order(SortOrder.DESC))
                .build();

        Page<EsLecturer> page = elasticsearchRestTemplate.queryForPage(searchQuery, EsLecturer.class, IndexCoordinates.of(Constant.INDEX_LECTURER));
        return Response.ok(page);
    }

    /**
     * 更新启用禁用状态
     *
     * @param map
     * @return
     */
    @PostMapping("/enableStatus/update")
    public boolean updateEnableStatus(@RequestBody Map<String, Object> map) {
        Long id = Long.valueOf(String.valueOf(map.get("id")));
        boolean enabled = Boolean.valueOf(String.valueOf(map.get("enabled")));
        Optional<EsLecturer> optional = lecturerRepository.findById(id);
        if (!optional.isPresent()) {
            return false;
        }
        EsLecturer lecturer = optional.get();

        HashMap<String, Object> params = new HashMap<>();
        params.put("status", enabled ? Constant.STATUS_UP : Constant.STATUS_DOWN);

        UpdateQuery updateQuery = UpdateQuery.builder(lecturer.getId())
                .withDocument(Document.from(params))
                .build();

        UpdateResponse updateResponse = elasticsearchRestTemplate.update(updateQuery, IndexCoordinates.of(Constant.INDEX_LECTURER));
        return UpdateResponse.Result.UPDATED.equals(updateResponse.getResult())
                // 未改变值也算更新成功
                || UpdateResponse.Result.NOOP.equals(updateResponse.getResult());

    }

}
