package com.yizhi.esearch.application.controller.api;

import com.yizhi.esearch.application.constant.Constant;
import com.yizhi.esearch.application.pojo.biz.EsResearch;
import com.yizhi.esearch.application.util.QueryHelper;
import com.yizhi.esearch.application.util.ResponseHelper;
import com.yizhi.util.application.domain.Response;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.elasticsearch.core.ElasticsearchRestTemplate;
import org.springframework.data.elasticsearch.core.mapping.IndexCoordinates;
import org.springframework.data.elasticsearch.core.query.Query;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @Author: shengchenglong
 * @Date: 2018/12/26 17:06
 */
@Api(tags = "调研搜索")
@RestController
@RequestMapping("/search/research")
public class ResearchController {

    @Autowired
    private ElasticsearchRestTemplate elasticsearchRestTemplate;

    @GetMapping("name/{keyword}/{pageNo}/{pageSize}")
    @ApiOperation(value = "调研搜索", response = EsResearch.class)
    public Response<List<EsResearch>> name(
            @ApiParam(name = "keyword", value = "搜索关键词") @PathVariable("keyword") String keyword,
            @ApiParam(name = "pageNo", value = "第几页") @PathVariable("pageNo") Integer pageNo,
            @ApiParam(name = "pageSize", value = "每页几条") @PathVariable("pageSize") Integer pageSize) {
        Query searchQuery = QueryHelper.commonPageSearchQueryForKeyword(keyword, EsResearch.class, pageNo, pageSize, "endTime", true, elasticsearchRestTemplate);
        Page<EsResearch> page = elasticsearchRestTemplate.queryForPage(searchQuery, EsResearch.class, IndexCoordinates.of(Constant.INDEX_RESEARCH));
        return ResponseHelper.ok(page);
    }

}
