package com.yizhi.esearch.application.controller.api;

import com.yizhi.esearch.application.constant.Constant;
import com.yizhi.esearch.application.pojo.biz.EsTrainingProject;
import com.yizhi.esearch.application.util.QueryHelper;
import com.yizhi.esearch.application.util.ResponseHelper;
import com.yizhi.util.application.domain.Response;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.elasticsearch.core.ElasticsearchRestTemplate;
import org.springframework.data.elasticsearch.core.mapping.IndexCoordinates;
import org.springframework.data.elasticsearch.core.query.Query;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @Author: shengchenglong
 * @Date: 2018/12/24 14:39
 */
@Api(tags = "培训项目搜索")
@RestController
@RequestMapping("/search/trainingProject")
public class TrainingProjectController {

    @Autowired
    private ElasticsearchRestTemplate elasticsearchRestTemplate;

    @GetMapping("name/{keyword}/{pageNo}/{pageSize}")
    @ApiOperation(value = "培训项目搜索", response = EsTrainingProject.class)
    public Response<List<EsTrainingProject>> name(
            @ApiParam(name = "keyword", value = "搜索关键词") @PathVariable("keyword") String keyword,
            @ApiParam(name = "pageNo", value = "第几页") @PathVariable("pageNo") Integer pageNo,
            @ApiParam(name = "pageSize", value = "每页几条") @PathVariable("pageSize") Integer pageSize) {
        Query searchQuery = QueryHelper.commonPageSearchQueryForKeyword(keyword, EsTrainingProject.class, pageNo, pageSize, "releaseTime", false, elasticsearchRestTemplate);
        Page<EsTrainingProject> page = elasticsearchRestTemplate.queryForPage(searchQuery, EsTrainingProject.class, IndexCoordinates.of(Constant.INDEX_TRAINING_PROJECT));
        return ResponseHelper.ok(page);
    }
}
