package com.yizhi.esearch.application.controller.remote;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.esearch.application.pojo.biz.EsBizKeyword;
import com.yizhi.esearch.application.repository.BizKeywordRepository;
import com.yizhi.esearch.application.util.QueryHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.elasticsearch.core.query.NativeSearchQueryBuilder;
import org.springframework.data.elasticsearch.core.query.Query;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ClassName RecommendController
 * @Description TODO
 * @Author shengchenglong
 * @DATE 2019-12-30 14:14
 * @Version 1.0
 */
@Slf4j
@RestController
@RequestMapping("/recommend/bizKeyword")
public class BizKeywordController {

    @Autowired
    private BizKeywordRepository bizKeywordRepository;

    /**
     * taskId
     * taskFieldValue
     * taskParamsType
     */
    @PostMapping("keywordPage")
    public Map<String, String> keywordPage(@RequestBody Map<String, String> map) {
        int pageNo = Integer.parseInt(map.get("pageNo"));
        int pageSize = Integer.parseInt(map.get("pageSize"));
        String contextJson = map.get("context");
        RequestContext context = JSON.parseObject(contextJson, RequestContext.class);
        ContextHolder.set(context);

        BoolQueryBuilder qb = QueryBuilders.boolQuery()
                .must(QueryBuilders.termQuery("recommendType", map.get("taskType").toLowerCase()));

        if (StringUtils.isNotEmpty(map.get("taskParamsValue"))) {
            qb.must(QueryBuilders.matchQuery("value", map.get("taskParamsValue")));
        }

        Query searchQuery = new NativeSearchQueryBuilder()
                .withQuery(qb)
                .withFilter(QueryHelper.getCommonQueryBuilderForContext())
                .withPageable(PageRequest.of(pageNo - 1, pageSize))
                .build();

        Page page = bizKeywordRepository.search(searchQuery);
        List<EsBizKeyword> list = page.getContent();

        Map<String, String> resMap = new HashMap<>();
        resMap.put("total", String.valueOf(page.getTotalElements()));
        if (!CollectionUtils.isEmpty(list)) {
            JSONArray jsonArray = new JSONArray();
            list.forEach(item -> {
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("taskId", item.getBizId());
                jsonObject.put("taskFieldValue", item.getValue());
                jsonObject.put("taskParamsType", "keyword");
                jsonArray.add(jsonObject);
            });
            resMap.put("record", jsonArray.toJSONString());
        }
        log.info(JSON.toJSONString(resMap));
        return resMap;
    }


}
