package com.yizhi.esearch.application.controller.remote;

import com.yizhi.esearch.application.constant.Constant;
import com.yizhi.esearch.application.pojo.system.EsOrganization;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.sort.SortBuilders;
import org.elasticsearch.search.sort.SortOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.elasticsearch.core.ElasticsearchRestTemplate;
import org.springframework.data.elasticsearch.core.mapping.IndexCoordinates;
import org.springframework.data.elasticsearch.core.query.GetQuery;
import org.springframework.data.elasticsearch.core.query.NativeSearchQueryBuilder;
import org.springframework.data.elasticsearch.core.query.Query;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;

/**
 * @ClassName OrganizationController
 * @Description TODO
 * @Author chengchenglong
 * @DATE 2019-04-18 16:47
 * @Version 1.0
 */
@RestController
@RequestMapping("/org")
public class OrganizationController {

    @Autowired
    private ElasticsearchRestTemplate elasticsearchTemplate;

    /**
     * 查询父级部门的名称集合
     * @param orgId
     * @return
     */
    @GetMapping("/parent/names")
    public List<String> getParentNames(@RequestParam("orgId") Long orgId) {
        GetQuery getQuery = new GetQuery(String.valueOf(orgId));
        EsOrganization org = elasticsearchTemplate.queryForObject(getQuery, EsOrganization.class);

        if (org != null) {
            int leftIndex = org.getLeftIndex();
            int rightIndex = org.getRightIndex();

            QueryBuilder leftIndexQB = QueryBuilders.rangeQuery("leftIndex").lte(leftIndex);
            QueryBuilder rightIndexQB = QueryBuilders.rangeQuery("rightIndex").gte(rightIndex);

            QueryBuilder queryBuilder = QueryBuilders.boolQuery()
                    .must(QueryBuilders.termQuery(Constant.FIELD_COMPANY_ID, org.getCompanyId()))
                    .must(QueryBuilders.termQuery(Constant.FIELD_DELETED, Constant.DELETED_NO))
                    .must(leftIndexQB)
                    .must(rightIndexQB);

            Query searchQuery = new NativeSearchQueryBuilder()
                    .withQuery(queryBuilder)
                    .withSort(SortBuilders.fieldSort("leftIndex").order(SortOrder.ASC))
                    // notice: 这里指定字段则只会查询相应的字段值，其余字段为null
                    .withFields("name")
                    .build();

            List<EsOrganization> parents = elasticsearchTemplate.queryForList(searchQuery, EsOrganization.class, IndexCoordinates.of("organization"));
            if (!CollectionUtils.isEmpty(parents)) {
                List<String> names = new ArrayList<>(parents.size() + 1);
                parents.forEach(item -> names.add(item.getName()));
                names.add(org.getName());
                return names;
            }
        }
        return null;
    }

}
