package com.yizhi.esearch.application.controller.remote;

import com.alibaba.fastjson.JSON;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.drools.application.constant.DroolsConstants;
import com.yizhi.drools.application.vo.drools.*;
import com.yizhi.drools.application.vo.manage.RuleItemVo;
import com.yizhi.drools.application.vo.manage.RuleVo;
import com.yizhi.esearch.application.eum.RecommendAccountRelationEnum;
import com.yizhi.esearch.application.eum.RecommendBusinessRelationEnum;
import com.yizhi.esearch.application.eum.RuleAndOrQbBuilder;
import com.yizhi.esearch.application.eum.RuleItemAndOrQbBuilder;
import com.yizhi.esearch.application.pojo.biz.EsDocRecommend;
import com.yizhi.esearch.application.pojo.system.EsAccount;
import com.yizhi.esearch.application.util.QueryHelper;
import lombok.extern.slf4j.Slf4j;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.sort.SortBuilders;
import org.elasticsearch.search.sort.SortOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.elasticsearch.core.ElasticsearchRestTemplate;
import org.springframework.data.elasticsearch.core.mapping.IndexCoordinates;
import org.springframework.data.elasticsearch.core.query.NativeSearchQueryBuilder;
import org.springframework.data.elasticsearch.core.query.Query;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.*;

/**
 * @ClassName RecommendController
 * @Description TODO
 * @Author shengchenglong
 * @DATE 2019-12-30 14:14
 * @Version 1.0
 */
@Slf4j
@RestController
@RequestMapping("/recommend/")
public class RecommendController {

    @Autowired
    private ElasticsearchRestTemplate elasticsearchRestTemplate;

    @PostMapping("/recommend4Manage")
    public BusinessVo4PortalResponse recommend4Manage(@RequestBody BusinessVo4PortalParam param) {
        log.info(" ------------------------- " + param);
        System.out.println(JSON.toJSONString(param));

        ContextHolder.set(param.getRequestContext());

        Map<String, QueryBuilder> business_qb_map = build_index_qb_map(param);
        return queryBusinessResult(param, business_qb_map);
    }


    @PostMapping("/recommend4Portal")
    public BusinessVo4PortalResponse recommend4Portal(@RequestBody BusinessVo4PortalParam param) {
        log.info("protal -- param: " + JSON.toJSONString(param));

        RequestContext context = param.getRequestContext();
        // 如果没有上下文，返回空
        if (context == null) {
            return new BusinessVo4PortalResponse();
        }
        ContextHolder.set(context);

        Map<String, QueryBuilder> business_qb_map = build_index_qb_map(param);

        // 没有策略
        if (CollectionUtils.isEmpty(business_qb_map)) {
            return new BusinessVo4PortalResponse();
        }

        return queryBusinessResult(param, business_qb_map);
    }

    @PostMapping("/recommend4ManageAccount")
    public AccountVo4ManageResponse recommend4ManageAccount(@RequestBody AccountVo4ManageParam param) {
        BoolQueryBuilder queryBuilder = buildQueryBuilder4Account(param.getRuleVos());
        if (queryBuilder == null) {
            return null;
        }
        queryBuilder.must(QueryBuilders.termQuery("companyId", param.getRequestContext().getCompanyId()));
        if (!StringUtils.isEmpty(param.getKeyword())) {
            queryBuilder.must(QueryBuilders.boolQuery()
                    .should(QueryBuilders.matchQuery("name", param.getKeyword()))
                    .should(QueryBuilders.matchQuery("fullName", param.getKeyword()))
                    .should(QueryBuilders.matchQuery("mobile", param.getKeyword())));
        }

        Query searchQuery = new NativeSearchQueryBuilder()
                .withPageable(PageRequest.of(param.getPageNo() - 1, param.getPageSize()))
                .withQuery(queryBuilder)
                .build();

        log.info("管理端 account - " + queryBuilder);
        AccountVo4ManageResponse response = new AccountVo4ManageResponse();

        Page<EsAccount> page = elasticsearchRestTemplate.queryForPage(searchQuery, EsAccount.class, IndexCoordinates.of("account"));
        List<AccountVo4Manage> accountVo4ManageList = new ArrayList<>();
        List<EsAccount> accountList = page.getContent();
        if (!CollectionUtils.isEmpty(accountList)) {
            accountVo4ManageList = new ArrayList<>(accountList.size());
            AccountVo4Manage accountVo4Manage;
            EsAccount account;
            for (int i = 0; i < accountList.size(); i++) {
                accountVo4Manage = new AccountVo4Manage();
                account = accountList.get(i);
                accountVo4Manage.setFullName(account.getFullName());
                accountVo4Manage.setId(Long.valueOf(account.getId()));
                accountVo4Manage.setMobile(account.getMobile());
                accountVo4Manage.setName(account.getName());
                accountVo4Manage.setNum(i + 1);
                accountVo4Manage.setPosition(account.getPosition());
                accountVo4ManageList.add(accountVo4Manage);
            }
        }

        response.setList(accountVo4ManageList);
        response.setPageNo(param.getPageNo());
        response.setPageSize(param.getPageSize());
        response.setTotal(Math.toIntExact(page.getTotalElements()));
        return response;
    }

    /**
     * 组装 business/account key - qb map
     *
     * @param param
     * @return
     */
    private Map<String, QueryBuilder> build_index_qb_map(BusinessVo4PortalParam param) {
        List<RuleVo> ruleVos = param.getRuleVos();
        Map<Long, List<RuleVo>> map = param.getStrategyRuleVoMap();

        Map<String, QueryBuilder> business_qb_map_rule;

        if (CollectionUtils.isEmpty(map)) {
            business_qb_map_rule = buildQueryBuilderMap4Manage(param, ruleVos);
        } else {
            business_qb_map_rule = buildQueryBuilderMap4Portal(param, map);
        }

        return business_qb_map_rule;
    }

    /**
     * 根据组装好的 qb map 查询业务数据
     *
     * @param param
     * @param business_qb_map
     * @return
     */
    private BusinessVo4PortalResponse queryBusinessResult(BusinessVo4PortalParam param, Map<String, QueryBuilder> business_qb_map) {
        // 没有策略
        if (CollectionUtils.isEmpty(business_qb_map)) {
            return new BusinessVo4PortalResponse();
        }

        business_qb_map.remove(DroolsConstants.BUSINESS_NAME_USER);

        BusinessVo4PortalResponse response = new BusinessVo4PortalResponse();
        response.setPageNo(param.getPageNo());
        response.setPageSize(param.getPageSize());
        response.setTotal(0);

        BoolQueryBuilder BizQb = QueryBuilders.boolQuery();
        business_qb_map.forEach((key, item) -> {
            BoolQueryBuilder currentBizQb = QueryBuilders.boolQuery()
                    .must(QueryBuilders.termQuery("recommendType", key.toLowerCase()))
                    .must(item);
            BizQb.should(currentBizQb);
        });

        BoolQueryBuilder qb = QueryBuilders.boolQuery();
        qb.must(BizQb);
        // 这里如果keywords里面有值，则是管理端的搜索，需要加上过滤
        if (!StringUtils.isEmpty(param.getKeyword())) {
            QueryBuilder keywordsQb = QueryBuilders.boolQuery()
                    .should(QueryBuilders.matchQuery("name", param.getKeyword().toLowerCase()))
                    .should(QueryBuilders.matchQuery("code", param.getKeyword().toLowerCase()));
            qb.must(keywordsQb);
        }

        Query searchQuery = new NativeSearchQueryBuilder()
                .withQuery(qb)
                .withFilter(QueryHelper.getCommonQueryBuilderForContext())
                // page从0开始
                .withPageable(PageRequest.of(param.getPageNo() - 1, param.getPageSize()))
                // notice: 先按评分倒排，再按业务要求排序
                .withSort(SortBuilders.scoreSort().order(SortOrder.DESC))
                // .withSort(isAsc ? SortBuilders.fieldSort(sortFiled).order(SortOrder.ASC) : SortBuilders.fieldSort(sortFiled).order(SortOrder.DESC))
                .build();

        Page<EsDocRecommend> page = elasticsearchRestTemplate.queryForPage(searchQuery, EsDocRecommend.class, IndexCoordinates.of("doc_recommend"));
        log.info(" --------------------------------- page " + JSON.toJSONString(page));
        List<EsDocRecommend> list = page.getContent();
        if (!CollectionUtils.isEmpty(list)) {
            List<BusinessVo4Portal> resList = new ArrayList<>(list.size());
            for (EsDocRecommend docRecommend : list) {
                BusinessVo4Portal businessVo4Portal = new BusinessVo4Portal();
                businessVo4Portal.setId(Long.valueOf(docRecommend.getRecommendId().split("_")[0]));
                businessVo4Portal.setType(docRecommend.getRecommendType());
                businessVo4Portal.setName(docRecommend.getName());
                businessVo4Portal.setImage(docRecommend.getImage());
                businessVo4Portal.setCode(docRecommend.getCode());
                resList.add(businessVo4Portal);
            }
            response.setList(resList);
            response.setTotal((int) page.getTotalElements());
        }
        return response;
    }

    /**
     * manage 组装 queryBuilder
     *
     * @param param
     * @param ruleVos
     * @return
     */
    private Map<String, QueryBuilder> buildQueryBuilderMap4Manage(BusinessVo4PortalParam param, List<RuleVo> ruleVos) {
        Map<String, QueryBuilder> business_qb_map_rule = new HashMap<>();
        Map<String, QueryBuilder> business_qb_map_ruleItem;

        RuleVo ruleVo;
        QueryBuilder qbCurrentLoop = null;
        if (CollectionUtils.isEmpty(ruleVos)){
            log.error("ruleVos参数为空");
            return business_qb_map_rule;
        }

        for (int i = 0; i < ruleVos.size(); i++) {
            ruleVo = ruleVos.get(i);

            // 如果条件规则是 用户 规则,继续
            if (ruleVo.getType().equals(DroolsConstants.RULE_TYPE_USER)) {
                continue;
            }

            List<RuleItemVo> ruleItemVosTmp = ruleVo.getList();
            String ruleAndOr = ruleVo.getAndOr();

            if (!CollectionUtils.isEmpty(ruleItemVosTmp)) {
                business_qb_map_ruleItem = new HashMap<>();
                for (int j = 0; j < ruleItemVosTmp.size(); j++) {
                    RuleItemVo itemVo = ruleItemVosTmp.get(j);

                    if (!StringUtils.isEmpty(itemVo.getValue())) {
                        itemVo.setValue(itemVo.getValue().toLowerCase());
                    }

                    qbCurrentLoop = business_qb_map_ruleItem.get(itemVo.getBusiness()) == null
                            ? QueryBuilders.boolQuery() : business_qb_map_ruleItem.get(itemVo.getBusiness());

                    // 如果business不为空，则是管理端的内容搜索，直接过滤掉非相关规则
                    if (!StringUtils.isEmpty(param.getBusiness())
                            && !param.getBusiness().equalsIgnoreCase(itemVo.getBusiness())) {
                        continue;
                    }

                    // 1. 如果还没有当前业务的queryBuilder
                    if (!business_qb_map_ruleItem.containsKey(itemVo.getBusiness())) {
                        // 如果当前查询所有
                        if ("all".equalsIgnoreCase(itemVo.getProperty())) {
                            qbCurrentLoop = QueryBuilders.boolQuery();
                        } else {
                            qbCurrentLoop = RecommendBusinessRelationEnum.valueOf(itemVo.getRelation().toUpperCase(Locale.ENGLISH))
                                    .build(itemVo);
                        }
                        business_qb_map_ruleItem.put(itemVo.getBusiness(), qbCurrentLoop);
                        continue;
                    }

                    // 2. 如果有当前业务的 queryBuilder
                    qbCurrentLoop = RuleItemAndOrQbBuilder.valueOf(itemVo.getAndOr().toUpperCase(Locale.ENGLISH))
                            .build(qbCurrentLoop, itemVo);
                    business_qb_map_ruleItem.put(itemVo.getBusiness(), qbCurrentLoop);
                }
                // 3. 如果是最后一次循环 组装上rule的andOr
                for (Map.Entry<String, QueryBuilder> stringQueryBuilderEntry : business_qb_map_ruleItem.entrySet()) {
                    if (business_qb_map_rule.containsKey(stringQueryBuilderEntry.getKey())) {
                        business_qb_map_rule.put(stringQueryBuilderEntry.getKey(),
                                RuleAndOrQbBuilder.valueOf(ruleAndOr.toUpperCase(Locale.ENGLISH))
                                        .build(business_qb_map_rule.get(stringQueryBuilderEntry.getKey()), qbCurrentLoop));
                    } else {
                        business_qb_map_rule.put(stringQueryBuilderEntry.getKey(), stringQueryBuilderEntry.getValue());
                    }
                }
            }
        }
        return business_qb_map_rule;
    }

    /**
     * portal 组装 queryBuilder
     *
     * @param param
     * @param map
     * @return
     */
    private Map<String, QueryBuilder> buildQueryBuilderMap4Portal(BusinessVo4PortalParam param, Map<Long, List<RuleVo>> map) {
        Map<String, QueryBuilder> business_qb_map_rule = new HashMap<>();
        Map<String, QueryBuilder> business_qb_map_ruleItem;

        RuleVo ruleVo;
        QueryBuilder qbCurrentLoop = null;

        for (Map.Entry<Long, List<RuleVo>> longListEntry : map.entrySet()) {
            List<RuleVo> ruleVos = longListEntry.getValue();

            boolean flag = containsCurrentAccount(ruleVos, param.getRequestContext());
            if (!flag) {
                continue;
            }

            for (int i = 0; i < ruleVos.size(); i++) {
                ruleVo = ruleVos.get(i);

                List<RuleItemVo> ruleItemVosTmp = ruleVo.getList();
                String ruleAndOr = ruleVo.getAndOr();

                if (!CollectionUtils.isEmpty(ruleItemVosTmp)) {
                    business_qb_map_ruleItem = new HashMap<>();
                    for (int j = 0; j < ruleItemVosTmp.size(); j++) {
                        RuleItemVo itemVo = ruleItemVosTmp.get(j);

                        if (!StringUtils.isEmpty(itemVo.getValue())) {
                            itemVo.setValue(itemVo.getValue().toLowerCase());
                        }

                        qbCurrentLoop = business_qb_map_ruleItem.get(itemVo.getBusiness()) == null
                                ? QueryBuilders.boolQuery() : business_qb_map_ruleItem.get(itemVo.getBusiness());

                        // 如果business不为空，则是管理端的内容搜索，直接过滤掉非相关规则
                        if (!StringUtils.isEmpty(param.getBusiness())
                                && !param.getBusiness().equalsIgnoreCase(itemVo.getBusiness())) {
                            continue;
                        }

                        // 1. 如果还没有当前业务的queryBuilder
                        if (!business_qb_map_ruleItem.containsKey(itemVo.getBusiness())) {
                            // 如果当前查询所有
                            if ("all".equalsIgnoreCase(itemVo.getProperty())) {
                                qbCurrentLoop = QueryBuilders.boolQuery();
                            } else {
                                qbCurrentLoop = RecommendBusinessRelationEnum.valueOf(itemVo.getRelation().toUpperCase(Locale.ENGLISH))
                                        .build(itemVo);
                            }
                            business_qb_map_ruleItem.put(itemVo.getBusiness(), qbCurrentLoop);
                            continue;
                        }

                        // 2. 如果有当前业务的 queryBuilder
                        qbCurrentLoop = RuleItemAndOrQbBuilder.valueOf(itemVo.getAndOr().toUpperCase(Locale.ENGLISH))
                                .build(qbCurrentLoop, itemVo);
                        business_qb_map_ruleItem.put(itemVo.getBusiness(), qbCurrentLoop);
                    }
                    // 3. 如果是最后一次循环 组装上rule的andOr
                    for (Map.Entry<String, QueryBuilder> stringQueryBuilderEntry : business_qb_map_ruleItem.entrySet()) {
                        if (business_qb_map_rule.containsKey(stringQueryBuilderEntry.getKey())) {
                            business_qb_map_rule.put(stringQueryBuilderEntry.getKey(),
                                    RuleAndOrQbBuilder.valueOf(ruleAndOr.toUpperCase(Locale.ENGLISH))
                                            .build(business_qb_map_rule.get(stringQueryBuilderEntry.getKey()), qbCurrentLoop));
                        } else {
                            business_qb_map_rule.put(stringQueryBuilderEntry.getKey(), stringQueryBuilderEntry.getValue());
                        }
                    }
                }
            }
        }
        return business_qb_map_rule;
    }

    /**
     * 查看当前策略策略下推荐的用户 是否 包含当前
     *
     * @param ruleVos
     * @param requestContext
     * @return
     */
    private boolean containsCurrentAccount(List<RuleVo> ruleVos, RequestContext requestContext) {
        BoolQueryBuilder finalQb = buildQueryBuilder4Account(ruleVos);
        if (finalQb == null) {
            return false;
        } else {
            finalQb.must(QueryBuilders.termQuery("id", String.valueOf(requestContext.getAccountId())))
                    .must(QueryBuilders.termQuery("companyId", requestContext.getCompanyId()));
        }
        Query searchQuery = new NativeSearchQueryBuilder()
                .withQuery(finalQb)
                .build();

        log.info("account - " + finalQb);
        // 查询当前用户是否符合 用户规则
        long count = elasticsearchRestTemplate.count(searchQuery, EsAccount.class);
        return count > 0;
    }

    /**
     * 获取用户的queryBuildervo
     *
     * @param ruleVos
     * @return
     */
    private BoolQueryBuilder buildQueryBuilder4Account(List<RuleVo> ruleVos) {
        log.info("  --  buildQueryBuilder4Account ", JSON.toJSONString(ruleVos));
        Map<String, QueryBuilder> business_qb_map_rule = new HashMap<>();
        Map<String, QueryBuilder> business_qb_map_ruleItem;

        RuleVo ruleVo;
        QueryBuilder qbCurrentLoop = null;

        for (int i = 0; i < ruleVos.size(); i++) {
            ruleVo = ruleVos.get(i);

            // 如果条件规则是 用户 规则,继续
            if (!ruleVo.getType().equals(DroolsConstants.RULE_TYPE_USER)) {
                continue;
            }

            List<RuleItemVo> ruleItemVosTmp = ruleVo.getList();
            String ruleAndOr = ruleVo.getAndOr();

            if (!CollectionUtils.isEmpty(ruleItemVosTmp)) {
                business_qb_map_ruleItem = new HashMap<>();
                for (int j = 0; j < ruleItemVosTmp.size(); j++) {
                    RuleItemVo itemVo = ruleItemVosTmp.get(j);

                    if (!StringUtils.isEmpty(itemVo.getValue())) {
                        itemVo.setValue(itemVo.getValue().toLowerCase());
                    }

                    qbCurrentLoop = business_qb_map_ruleItem.get(itemVo.getBusiness()) == null
                            ? QueryBuilders.boolQuery() : business_qb_map_ruleItem.get(itemVo.getBusiness());

                    // 1. 如果还没有当前业务的queryBuilder
                    if (!business_qb_map_ruleItem.containsKey(itemVo.getBusiness())) {
                        // 如果当前查询所有
                        if ("all".equalsIgnoreCase(itemVo.getProperty())) {
                            qbCurrentLoop = QueryBuilders.boolQuery();
                        } else {
                            qbCurrentLoop = RecommendAccountRelationEnum.valueOf(itemVo.getRelation().toUpperCase(Locale.ENGLISH))
                                    .build(itemVo);
                        }
                        business_qb_map_ruleItem.put(itemVo.getBusiness(), qbCurrentLoop);
                        continue;
                    }

                    // 2. 如果有当前业务的 queryBuilder
                    qbCurrentLoop = RuleItemAndOrQbBuilder.valueOf(itemVo.getAndOr().toUpperCase(Locale.ENGLISH))
                            .build(qbCurrentLoop, itemVo);
                    business_qb_map_ruleItem.put(itemVo.getBusiness(), qbCurrentLoop);
                }
                // 3. 如果是最后一次循环 组装上rule的andOr
                for (Map.Entry<String, QueryBuilder> stringQueryBuilderEntry : business_qb_map_ruleItem.entrySet()) {
                    if (business_qb_map_rule.containsKey(stringQueryBuilderEntry.getKey())) {
                        business_qb_map_rule.put(stringQueryBuilderEntry.getKey(),
                                RuleAndOrQbBuilder.valueOf(ruleAndOr.toUpperCase(Locale.ENGLISH))
                                        .build(business_qb_map_rule.get(stringQueryBuilderEntry.getKey()), qbCurrentLoop));
                    } else {
                        business_qb_map_rule.put(stringQueryBuilderEntry.getKey(), stringQueryBuilderEntry.getValue());
                    }
                }
            }
        }
        QueryBuilder accountQb = business_qb_map_rule.get(DroolsConstants.BUSINESS_NAME_USER);
        if (accountQb == null) {
            return null;
        }
        return QueryBuilders.boolQuery().must(accountQb);
    }
}
