package com.yizhi.esearch.application.scheduler;


import com.yizhi.esearch.application.pojo.biz.EsBizKeyword;
import com.yizhi.esearch.application.pojo.biz.EsDocRecommend;
import com.yizhi.esearch.application.repository.BizKeywordRepository;
import com.yizhi.esearch.application.repository.DocRecommendRepository;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.elasticsearch.core.ElasticsearchRestTemplate;
import org.springframework.data.elasticsearch.core.SearchScrollHits;
import org.springframework.data.elasticsearch.core.mapping.IndexCoordinates;
import org.springframework.data.elasticsearch.core.query.NativeSearchQueryBuilder;
import org.springframework.data.elasticsearch.core.query.Query;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

/**
 * @ClassName KeywordScheduler
 * @Description TODO
 * @Author shengchenglong
 * @DATE 2020-03-11 16:40
 * @Version 1.0
 */
@Component
@Log4j2
public class KeywordScheduler {

    @Autowired
    private DocRecommendRepository docRecommendRepository;
    @Autowired
    private BizKeywordRepository bizKeywordRepository;
    @Autowired
    private ElasticsearchRestTemplate elasticsearchRestTemplate;
    @Value("${needInit}")
    private String needInit;


    //    @PostConstruct
    @Scheduled(cron = "0 0/5 * * * ?")
    public void run() throws ParseException {
        Date date = new Date();
        if ("yes".equalsIgnoreCase(needInit)) {
            date = DateUtils.parseDate("2000-01-01 00:00:00", "yyyy-MM-dd HH:mm:ss");
        }
        run(date);
    }

    public void run(Date date) {

        String time = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(date);
        log.info("-------------------------------------------- 同步关键字，开始 -----------------------------------------------");
        QueryBuilder qb = QueryBuilders.rangeQuery("@timestamp").gte(DateUtils.addMinutes(date, -6).getTime());
        Query searchQuery = new NativeSearchQueryBuilder().withQuery(qb).build();
        IndexCoordinates index = IndexCoordinates.of("doc_recommend");
        SearchScrollHits<EsDocRecommend> scroll = elasticsearchRestTemplate.searchScrollStart(1000, searchQuery, EsDocRecommend.class, index);
        String scrollId = scroll.getScrollId();
        while (scroll.hasSearchHits()) {
            scrollId = scroll.getScrollId();
            insert(scroll, time);
            scroll = elasticsearchRestTemplate.searchScrollContinue(scrollId, 1000, EsDocRecommend.class, index);

        }
        log.info("-------------------------------------------- 同步关键字，结束 -----------------------------------------------");
    }

    private void insert(SearchScrollHits<EsDocRecommend> scroll, String time) {
        List<EsBizKeyword> bizKeywords = new ArrayList<>();

        scroll.getSearchHits().forEach(a -> {
            EsDocRecommend doc = a.getContent();
            String keyword = doc.getKeyword();
            String[] keywords = null;
            if (StringUtils.isNotEmpty(keyword)) {

                keywords = keyword.split(",");
                for (String s : keywords) {
                    EsBizKeyword bizKeyword = new EsBizKeyword();
                    bizKeyword.setId(doc.getRecommendType().concat("_").concat(doc.getSiteId()).concat("_").concat(s));
                    bizKeyword.setBizId(doc.getId().split("_")[0]);
                    bizKeyword.setCompanyId(doc.getCompanyId());
                    bizKeyword.setDeleted(doc.getDeleted());
//                    bizKeyword.setKwValue(s);
                    bizKeyword.setKw_value(s);
                    bizKeyword.setKw_value(s);
                    bizKeyword.setSiteId(doc.getSiteId());
                    bizKeyword.setStatus(doc.getStatus());
                    bizKeyword.setValue(s);
                    bizKeyword.setRecommendType(doc.getRecommendType());
                    bizKeywords.add(bizKeyword);
                }
            }

        });

        if (!CollectionUtils.isEmpty(bizKeywords)) {
            Iterable<EsBizKeyword> it = bizKeywordRepository.saveAll(bizKeywords);
            Iterator<EsBizKeyword> itra = it.iterator();
            while (itra.hasNext()) {
                log.info(time + "   " + itra.next().toString());
            }
        }
    }
}
