package com.yizhi.esearch.application.service.using;

import com.yizhi.esearch.application.pojo.system.EsOrganization;
import com.yizhi.esearch.application.repository.OrganizationRepository;
import com.yizhi.esearch.application.util.OrgIndex;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Optional;

/**
 * @ClassName OrgIndexServiceUsing
 * @Description OrgIndexServiceUsing
 * @Author shengchenglong
 * @DATE 2020-06-10 10:48
 * @Version 1.0
 */
@Component
public class OrgIndexServiceUsing {

    @Autowired
    private OrganizationRepository organizationRepository;

    /**
     * 获取一个 部门的 orgIndex
     *
     * @param orgId
     * @return
     */
    public OrgIndex getOne(Long orgId) {
        Optional<EsOrganization> optional = organizationRepository.findById(orgId);
        if (optional.isPresent()) {
            return new OrgIndex(optional.get().getLeftIndex(), optional.get().getRightIndex());
        }
        return null;
    }

    /**
     * 获取一个部门 子集 的 orgIndex queryBuilder
     *
     * @param orgId
     * @return
     */
    public QueryBuilder get4ContainingChildren(Long orgId) {
        OrgIndex orgIndex = getOne(orgId);
        if (orgIndex == null) {
            return null;
        }
        return QueryBuilders.boolQuery()
                .must(QueryBuilders.rangeQuery("orgLeftIndex").gte(orgIndex.getLeftIndex()))
                .must(QueryBuilders.rangeQuery("orgRightIndex").lte(orgIndex.getRightIndex()));
    }
}
