package com.yizhi.esearch.application.util;


import net.sf.cglib.beans.BeanCopier;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @ClassName BeanUtils
 * @Description 高性能，高质量的 bean copy
 * @Author chengchenglong
 * @DATE 2019-04-17 15:11
 * @Version 1.0
 */
public class BeanUtils {

    /**
     * 缓存BeanCopier，节省创建时间，使用ConcurrentHashMap来保证同步
     */
    public static final Map<String, BeanCopier> BEAN_COPIER_MAP = new ConcurrentHashMap<>();

    public static void cpoy(Object origin, Object target) {
        BeanCopier beanCopier = getBeanCopier(origin.getClass(), target.getClass());
        if (beanCopier != null) {
            beanCopier.copy(origin, target, null);
        }
    }

    /**
     * 获取BeanCopier实例
     *
     * @param origin
     * @param target
     * @return
     */
    public static BeanCopier getBeanCopier(Class origin, Class target) {
        String key = origin.getName() + target.getName();
        if (BEAN_COPIER_MAP.containsKey(key)) {
            return BEAN_COPIER_MAP.get(key);
        }
        BeanCopier beanCopier = BeanCopier.create(origin, target, false);
        BEAN_COPIER_MAP.put(key, beanCopier);
        return beanCopier;
    }
}
