package com.fulan.esearch;

import com.yizhi.esearch.application.constant.Constant;
import com.yizhi.esearch.application.pojo.biz.EsCourse;
import com.yizhi.esearch.application.repository.CourseRepository;
import com.yizhi.esearch.application.util.ResponseHelper;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.elasticsearch.core.ElasticsearchOperations;
import org.springframework.data.elasticsearch.core.ElasticsearchRestTemplate;
import org.springframework.data.elasticsearch.core.SearchHits;
import org.springframework.data.elasticsearch.core.SearchScrollHits;
import org.springframework.data.elasticsearch.core.document.Document;
import org.springframework.data.elasticsearch.core.mapping.IndexCoordinates;
import org.springframework.data.elasticsearch.core.query.NativeSearchQueryBuilder;
import org.springframework.data.elasticsearch.core.query.Query;
import org.springframework.data.elasticsearch.core.query.UpdateQuery;
import org.springframework.data.elasticsearch.core.query.UpdateResponse;
import org.springframework.test.context.junit4.SpringRunner;

import java.util.HashMap;
import java.util.Map;

/**
 * ES7.x 测试类
 */
@RunWith(SpringRunner.class)
@SpringBootTest
public class CloudEsearch7ApplicationTests {

    @Autowired
    private ElasticsearchRestTemplate elasticsearchRestTemplate;
    @Autowired
    private ElasticsearchOperations elasticsearchOperations;
    @Autowired
    private CourseRepository courseRepository;

    @Test
    public void testSearch() {
        QueryBuilder queryBuilder = QueryBuilders.matchQuery("name", "课程");

        Query query = new NativeSearchQueryBuilder()
                .withPageable(PageRequest.of(0, 1000))
                .withQuery(queryBuilder)
                .build();

        SearchHits<EsCourse> searchHits = elasticsearchRestTemplate.search(query, EsCourse.class);
        // 这个是总条数
        searchHits.getTotalHits();

        Page<EsCourse> page = elasticsearchRestTemplate.queryForPage(query, EsCourse.class, IndexCoordinates.of("course"));
        System.out.println(ResponseHelper.ok(page));
    }



    /**
     * 测试scroll
     */
    @Test
    public void testScroll() {
        Query query = new NativeSearchQueryBuilder()
                .build();

        SearchScrollHits<EsCourse> searchScrollHits = elasticsearchRestTemplate.searchScrollStart(1000, query, EsCourse.class, IndexCoordinates.of("course"));
        String scrollId = searchScrollHits.getScrollId();

        searchScrollHits = elasticsearchRestTemplate.searchScrollContinue(scrollId, 1000, EsCourse.class, IndexCoordinates.of("course"));
        searchScrollHits = elasticsearchRestTemplate.searchScrollContinue(scrollId, 1000, EsCourse.class, IndexCoordinates.of("course"));
        searchScrollHits = elasticsearchRestTemplate.searchScrollContinue(scrollId, 1000, EsCourse.class, IndexCoordinates.of("course"));


    }

    @Test
    public void testUpdateQuery() {
        Map<String, Object> params = new HashMap<>();
        params.put("name", "1231fdsafdsafds23");
        UpdateQuery updateQuery = UpdateQuery.builder("1224671968932433920")
//                .withParams(params)
                .withDocument(Document.from(params))
                .build();

        UpdateResponse updateResponse = elasticsearchRestTemplate.update(updateQuery, IndexCoordinates.of(Constant.INDEX_COURSE));
        System.out.println(updateResponse.getResult());
        System.out.println(UpdateResponse.Result.UPDATED);
        System.out.println(UpdateResponse.Result.UPDATED.equals(updateResponse.getResult()));
    }




}

